@@ -10,7 +10,7 @@ use Carp;
 
 
 
-our $VERSION = "0.026";
+our $VERSION = "0.027";
 
 =head1 NAME
 
@@ -118,7 +118,7 @@ In I<convenience mode> this is ignored. E.g. a C<Times 20> font
 will be of equal size in the two systems,
 
 In I<Pango conformance mode> you would need to specify a font size of
-C<15000> to get a 20pt font.
+C<15360> to get a 20pt font.
 
 =head1 SUPPORTED MARKUP
 
@@ -398,6 +398,7 @@ sub set_text {
 	  color => $self->{_currentcolor},
 	  base  => 0,
 	} ];
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -764,6 +765,7 @@ sub set_markup {
 
     # Store content.
     $self->{_content} = \@content;
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -825,6 +827,7 @@ sub set_font_description {
 	}
     }
     $self->{_currentcolor} = $description->{color} || "black";
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -863,6 +866,7 @@ Implementation note: Only alignment is implemented.
 sub set_width {
     my ( $self, $width ) = @_;
     $self->{_width} = $self->{_pu2px}->($width);
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -896,6 +900,7 @@ Implementation note: Height restrictions are not yet implemented.
 sub set_height {
     my ( $self, $height ) = @_;
     $self->{_height} = $self->{_pu2px}->($height);
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -1079,6 +1084,7 @@ Not yet implemented.
 sub set_spacing {
     my ( $self, $spacing ) = @_;
     $self->{_currentspacing} = $self->{_pu2px}->($spacing);
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -1121,6 +1127,7 @@ Not yet implemented.
 sub set_line_spacing {
     my ( $self, $factor ) = @_;
     $self->{_currentlinespacing} = $factor;
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -1212,6 +1219,7 @@ sub set_alignment {
     else {
 	croak("Invalid alignment: $align");
     }
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -1343,19 +1351,20 @@ Computes the logical and ink extents of the layout.
 
 Logical extents are usually what you want for positioning things.
 
-Return value is an array (in list context) or an array ref (in scalar
-context) containing two hash refs with 4 values: C<x>, C<y>, C<width>,
-and C<height>. The first reflects the ink extents, the second the
-logical extents.
+Return value in scalar context is a hash ref with 4 values:
+ C<x>, C<y>, C<width>, and C<height>
+describing the logical extents of the layout.
+In list context an array of two hashrefs is returned.
+The first reflects the ink extents, the second the logical extents.
 
-C<x> will reflect the offset when text is centered or right aligned.
-It will be zero for left aligned text. For right aligned text, it will
-be the width of the layout.
+In the extents, C<x> will reflect the offset when text is centered or
+right aligned. It will be zero for left aligned text. For right
+aligned text, it will be the width of the layout.
 
 C<y> will reflect the offset when text is centered vertically or
 bottom aligned. It will be zero for top aligned text.
 
-Implementation note: Since the PDF::API support layer cannot calculate ink,
+Implementation note: If the PDF::API support layer cannot calculate ink,
 this function returns two identical extents.
 
 =back
@@ -1364,10 +1373,17 @@ this function returns two identical extents.
 
 sub get_extents {
     my ( $self ) = @_;
+    my $need_ink = wantarray;
 
-    my @bb = @{$self->get_bbox};
-    my $res = { x => $bb[0], y => 0, width => $bb[2], height => $bb[3]-$bb[1] };
-    return wantarray ? ( $res, $res ) : [ $res, $res ];
+    my @bb = @{ $self->get_bbox($need_ink) };
+    my $res = { x     => $bb[0], y      => 0,
+		width => $bb[2], height => $bb[3]-$bb[1] };
+    my $ink = $res;
+    if ( @bb > 4 ) {
+	$ink = { x     => $bb[4], y      => $bb[5],
+		 width => $bb[6], height => $bb[7]-$bb[5] };
+    }
+    return $need_ink ? ( $ink, $res ) : $res;
 }
 
 =over
@@ -1383,9 +1399,17 @@ Same as get_extents, but using device units.
 sub get_pixel_extents {
     my ( $self ) = @_;
 
-    my @bb = @{$self->get_pixel_bbox};
-    my $res = { x => $bb[0], y => 0, width => $bb[2], height => $bb[3]-$bb[1] };
-    return wantarray ? ( $res, $res ) : [ $res, $res ];
+    my $need_ink = wantarray;
+
+    my @bb = @{ $self->get_pixel_bbox($need_ink) };
+    my $res = { x     => $bb[0], y      => 0,
+		width => $bb[2], height => $bb[3]-$bb[1] };
+    my $ink = $res;
+    if ( @bb > 4 ) {
+	$ink = { x     => $bb[4], y      => $bb[5],
+		 width => $bb[6], height => $bb[7]-$bb[5] };
+    }
+    return $need_ink ? ( $ink, $res ) : $res;
 }
 
 =over
@@ -1406,7 +1430,7 @@ returned.
 
 sub get_size {
     my ( $self ) = @_;
-    my $e = ($self->get_extents)[1];
+    my $e = $self->get_extents;
     wantarray
       ? return ( $e->{width}, $e->{height} )
       : return { width => $e->{width}, height => $e->{height} };
@@ -1424,7 +1448,7 @@ Same as get_size().
 
 sub get_pixel_size {
     my ( $self ) = @_;
-    my $e = ($self->get_pixel_extents)[1];
+    my $e = $self->get_pixel_extents;
     wantarray
       ? return ( $e->{width}, $e->{height} )
       : return { width => $e->{width}, height => $e->{height} };
@@ -1516,6 +1540,7 @@ Sets the size for the current font.
 sub set_font_size {
     my ( $self, $size ) = @_;
     $self->{_currentsize} = $size;
+    delete( $self->{_bbcache} );
 }
 
 =over
@@ -1567,19 +1592,33 @@ bb[3] = ascend is a positive value
 
 NOTE: Some fonts do not include accents on capital letters in the ascend.
 
+If an argument is supplied and true, get_bbox() will attempt to
+calculate the ink extents as well, and add these as another set of 4
+elements,
+
+In list context returns the array of values, in scalar context an
+array ref.
+
 =back
 
 =cut
 
 sub get_pixel_bbox {
-    my ( $self ) = @_;
-    my $res = $self->bbox;
+    my ( $self, $all ) = @_;
+    my $res;
+    if ( $self->{_bbcache}
+	 && @{ $self->{_bbcache} } == ($all ? 8 : 4) ) {
+	$res = $self->{_bbcache};
+    }
+    else {
+	$res = $self->{_bbcache} = $self->bbox($all);
+    }
     wantarray ? @$res : $res;
 }
 
 sub get_bbox {
-    my ( $self ) = @_;
-    my @res = map { $self->{_px2pu}->($_) } ( $self->get_pixel_bbox );
+    my ( $self, $all ) = @_;
+    my @res = map { $self->{_px2pu}->($_) } ( $self->get_pixel_bbox($all) );
     wantarray ? @res : \@res;
 }
 
@@ -1626,6 +1665,7 @@ sub set_pango_mode {
 
     $self->{_px2pu} = \&px2pu;
     $self->{_pu2px} = \&pu2px;
+    delete( $self->{_bbcache} );
     return $self->{_pango_scale} = 0;
 }
 
